"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.packageVersion = void 0;
/**
 * @internal
 */
exports.packageVersion = '2.10.6';
//# sourceMappingURL=version.js.map