#!/usr/bin/env node
"use strict";
/**
 * @license
 * Copyright 2023 Google Inc.
 * SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
const CLI_js_1 = require("./CLI.js");
void new CLI_js_1.CLI().run(process.argv);
//# sourceMappingURL=main-cli.js.map